library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.ALL;

package rtm_detector_pkg is

   type t_RTMM is (RTMM_ERROR,
                   RTMM_V1,
                   RESERVED0,
                   RESERVED1,
                   RESERVED2,
                   RESERVED3,
                   RESERVED4,
                   RESERVED5);

   type t_RTMP is (RTMP_ERROR,
                   RTMP_BLOCKING_V1,
                   RTMP_RS485_V1,
                   RESERVED0,
                   RESERVED1,
                   RESERVED2,
                   RESERVED3,
                   RESERVED4);

   constant c_RTMM_ERROR       : UNSIGNED(2 downto 0) := "111";
   constant c_RTMM_V1          : UNSIGNED(2 downto 0) := "110";
   constant c_RTMM_RESERVED0   : UNSIGNED(2 downto 0) := "101";
   constant c_RTMM_RESERVED1   : UNSIGNED(2 downto 0) := "100";
   constant c_RTMM_RESERVED2   : UNSIGNED(2 downto 0) := "011";
   constant c_RTMM_RESERVED3   : UNSIGNED(2 downto 0) := "010";
   constant c_RTMM_RESERVED4   : UNSIGNED(2 downto 0) := "001";
   constant c_RTMM_RESERVED5   : UNSIGNED(2 downto 0) := "000";

   constant c_RTMP_ERROR       : UNSIGNED(2 downto 0) := "111";
   constant c_RTMP_BLOCKING_V1 : UNSIGNED(2 downto 0) := "110";
   constant c_RTMP_RS485_V1    : UNSIGNED(2 downto 0) := "101";
   constant c_RTMP_RESERVED0   : UNSIGNED(2 downto 0) := "100";
   constant c_RTMP_RESERVED1   : UNSIGNED(2 downto 0) := "011";
   constant c_RTMP_RESERVED2   : UNSIGNED(2 downto 0) := "010";
   constant c_RTMP_RESERVED3   : UNSIGNED(2 downto 0) := "001";
   constant c_RTMP_RESERVED4   : UNSIGNED(2 downto 0) := "000";

   function f_UNSIGNED(identifier: t_RTMM) return UNSIGNED;
   function f_UNSIGNED(identifier: t_RTMP) return UNSIGNED;

   function f_RTMM(bit_code: UNSIGNED(2 downto 0)) return t_RTMM;
   function f_RTMP(bit_code: UNSIGNED(2 downto 0)) return t_RTMP;

end rtm_detector_pkg;

package body rtm_detector_pkg is

   --! @brief             translation function from t_rtmp to unsigned
   --! @param identifier  t_rtmp type to be translated
   function f_UNSIGNED(identifier: t_RTMM) return UNSIGNED is
      variable v_return   : UNSIGNED(2 downto 0);
   begin
      v_return := c_RTMM_ERROR;
      case identifier is
         when RTMM_ERROR =>
            null;
         when RTMM_V1    =>
            v_return := c_RTMM_V1;
         when RESERVED0  =>
            v_return := c_RTMM_RESERVED0;
         when RESERVED1  =>
            v_return := c_RTMM_RESERVED1;
         when RESERVED2  =>
            v_return := c_RTMM_RESERVED2;
         when RESERVED3  =>
            v_return := c_RTMM_RESERVED3;
         when RESERVED4  =>
            v_return := c_RTMM_RESERVED4;
         when RESERVED5  =>
            v_return := c_RTMM_RESERVED5;
         when others     =>
            null;
      end case;
      return v_return;
   end f_UNSIGNED;

   --! @brief             translation function from t_rtmp to unsigned
   --! @param identifier  t_rtmp type to be translated
   function f_UNSIGNED(identifier: t_RTMP) return UNSIGNED is
      variable v_return   : UNSIGNED(2 downto 0);
   begin
      v_return := c_RTMP_ERROR;
      case identifier is
         when RTMP_ERROR       =>
            null;
         when RTMP_BLOCKING_V1 =>
            v_return := c_RTMP_BLOCKING_V1;
         when RTMP_RS485_V1    =>
            v_return := c_RTMP_RS485_V1;
         when RESERVED0        =>
            v_return := c_RTMP_RESERVED0;
         when RESERVED1        =>
            v_return := c_RTMP_RESERVED1;
         when RESERVED2        =>
            v_return := c_RTMP_RESERVED2;
         when RESERVED3        =>
            v_return := c_RTMP_RESERVED3;
         when RESERVED4        =>
            v_return := c_RTMP_RESERVED4;
         when others           =>
            null;
      end case;
      return v_return;
   end f_UNSIGNED;

   --! @brief           translation function from unsigned to t_RTMM
   --! @param bit_code  bit_code to be translated
   function f_RTMM(bit_code: UNSIGNED(2 downto 0)) return t_RTMM is
      variable v_return  : t_RTMM;
   begin
      v_return := RTMM_ERROR;
      case bit_code is
         when c_RTMM_ERROR     =>
            null;
         when c_RTMM_V1        =>
            v_return := RTMM_V1;
         when c_RTMM_RESERVED0 =>
            v_return := RESERVED0;
         when c_RTMM_RESERVED1 =>
            v_return := RESERVED1;
         when c_RTMM_RESERVED2 =>
            v_return := RESERVED2;
         when c_RTMM_RESERVED3 =>
            v_return := RESERVED3;
         when c_RTMM_RESERVED4 =>
            v_return := RESERVED4;
         when c_RTMM_RESERVED5 =>
            v_return := RESERVED5;
         when others           =>
            null;
      end case;
      return v_return;
   end f_RTMM;

   --! @brief           translation function from unsigned to t_RTMP
   --! @param bit_code  bit_code to be translated
   function f_RTMP(bit_code: UNSIGNED(2 downto 0)) return t_RTMP is
      variable v_return  : t_RTMP;
   begin
      v_return := RTMP_ERROR;
      case bit_code is
         when c_RTMP_ERROR     =>
            null;
         when c_RTMP_BLOCKING_V1 =>
            v_return := RTMP_BLOCKING_V1;
         when c_RTMP_RS485_V1    =>
            v_return := RTMP_RS485_V1;
         when c_RTMP_RESERVED0 =>
            v_return := RESERVED0;
         when c_RTMP_RESERVED1 =>
            v_return := RESERVED1;
         when c_RTMP_RESERVED2 =>
            v_return := RESERVED2;
         when c_RTMP_RESERVED3 =>
            v_return := RESERVED3;
         when c_RTMP_RESERVED4 =>
            v_return := RESERVED4;
         when others           =>
            null;
      end case;
      return v_return;
   end f_RTMP;

end rtm_detector_pkg;
