
//`def IMAGE2


module top_SP605_reboot (

      RESET,
      
      GCLK0,       // 27MHz clock

      REBOOT,     // re-boot command (active on high to low transition)
      
      ERASE_00,
      ERASE_10,
      ERASE_11,
      
      SPI_CS,
      SPI_CLK,
      SPI_DI,
      
      ICAP_DOUT_BITWISE_XOR,

      
      LED                                                                          
      );


input          RESET;

input          GCLK0;


input          REBOOT;
input          ERASE_00;
input          ERASE_10;
input          ERASE_11;

output         SPI_CS;
output         SPI_CLK;
output         SPI_DI;

output         ICAP_DOUT_BITWISE_XOR;


wire    clk_27MHz; 
assign  clk_27MHz = GCLK0;


wire       ICAP_BUSY;

wire    [15:0] icap_dout;

output   [3:0] LED;
reg      [3:0] LED;

wire           done_winbond;

reg     [20:0] count;
reg      [4:0] sub_count;

reg     [7:0] pattern;

reg      [4:0] q;
reg      [4:0] e0;
reg      [4:0] e1;
reg      [4:0] e2;

reg            reboot_ff;
reg            erase_00_ff;
reg            erase_10_ff;
reg            erase_11_ff;

reg      [1:0] clk_div;
wire           clk_13MHz;

assign clk_13MHz=clk_div[0];

// Slow the clock down to meet ICAP MINPERIOD Spec of 50 ns.

always @(posedge clk_27MHz)
begin
      clk_div = clk_div + 1;
end


ICAP_SP605_reboot ICAP_SP6_reboot_inst
   (
    .CLK                    (clk_13MHz),
    .MBT_RESET              (RESET),
    .MBT_REBOOT             (reboot_ff),
    .MBT_BUSY               (ICAP_BUSY),
    .ICAP_DOUT              (icap_dout) 
   );


winbond_fsm winbon_fsm_inst(
    .CLK            (clk_13MHz),
    .RESET          (RESET),
    .ERASE_00_0000  (erase_00_ff),
    .ERASE_10_0000  (erase_10_ff),
    .ERASE_10_1000  (erase_11_ff),
    .SPI_CS         (SPI_CS),
    .SPI_CLK        (SPI_CLK),
    .SPI_DI         (SPI_DI),
    .DONE           (done_winbond)
  );
  




assign ICAP_DOUT_BITWISE_XOR = ^icap_dout;

always@(posedge clk_13MHz)
begin
      q[0] <= REBOOT;
      q[1] <= q[0];
      q[2] <= q[1];
      q[3] <= q[2];
      q[4] <= q[3];
      reboot_ff <= (q[4] && (!q[3]) && (!q[2]) && (!q[1]) );
end

always@(posedge clk_13MHz)
begin
      e0[0] <= ERASE_00;
      e0[1] <= e0[0];
      e0[2] <= e0[1];
      e0[3] <= e0[2];
      e0[4] <= e0[3];
      erase_00_ff <= (e0[4] && (!e0[3]) && (!e0[2]) && (!e0[1]) );
end


always@(posedge clk_13MHz)
begin
      e1[0] <= ERASE_10;
      e1[1] <= e1[0];
      e1[2] <= e1[1];
      e1[3] <= e1[2];
      e1[4] <= e1[3];
      erase_10_ff <= (e1[4] && (!e1[3]) && (!e1[2]) && (!e1[1]) );
end

always@(posedge clk_13MHz)
begin
      e2[0] <= ERASE_11;
      e2[1] <= e2[0];
      e2[2] <= e2[1];
      e2[3] <= e2[2];
      e2[4] <= e2[3];
      erase_11_ff <= (e2[4] && (!e2[3]) && (!e2[2]) && (!e2[1]) );
end

     
       
always@(posedge clk_13MHz)
begin
   LED[0] <= pattern[0];
   LED[1] <= pattern[1];
   LED[2] <= pattern[2];
   LED[3] <= pattern[3];
end                                      


always@(posedge clk_13MHz)
begin
   count <= count + 2;
end
               
always@(posedge clk_13MHz)
begin
   if (!count)
   begin
      if (sub_count == 5'b10111) 
         sub_count <= 0;
      else
         sub_count <= sub_count + 1;
    end
end               

   always@(posedge clk_13MHz)
   begin
      if (!count)
      begin
      
   
         if      (sub_count == 5'b00000)  pattern <= 4'b0001; 
         else if (sub_count == 5'b00001)  pattern <= 4'b0010; 
         else if (sub_count == 5'b00010)  pattern <= 4'b0100; 
         else if (sub_count == 5'b00011)  pattern <= 4'b1000; 
         else if (sub_count == 5'b00100)  pattern <= 4'b1000; 
         else if (sub_count == 5'b00101)  pattern <= 4'b0100; 
         else if (sub_count == 5'b00110)  pattern <= 4'b0010; 
         else if (sub_count == 5'b00111)  pattern <= 4'b0001; 

         else if (sub_count == 5'b01000)  pattern <= 4'b0000; 
         else if (sub_count == 5'b01001)  pattern <= 4'b0000; 
         else if (sub_count == 5'b01010)  pattern <= 4'b0000; 
         else if (sub_count == 5'b01011)  pattern <= 4'b0000; 
         else if (sub_count == 5'b01100)  pattern <= 4'b0000; 
         else if (sub_count == 5'b01101)  pattern <= 4'b0000;   // 4'b1111;
         else if (sub_count == 5'b01110)  pattern <= 4'b0000;   // 4'b1111; 
         else if (sub_count == 5'b01111)  pattern <= 4'b1111;   // 4'b0000; 
 
         else if (sub_count == 5'b10000)  pattern <= 4'b1111;   // 4'b0000;
         else if (sub_count == 5'b10001)  pattern <= 4'b1111;   // 4'b1111; 
         else if (sub_count == 5'b10010)  pattern <= 4'b0000;   // 4'b1111; 
         else if (sub_count == 5'b10011)  pattern <= 4'b0000; 
         else if (sub_count == 5'b10100)  pattern <= 4'b0000; 
         else if (sub_count == 5'b10101)  pattern <= 4'b0000; 
         else if (sub_count == 5'b10110)  pattern <= 4'b0000; 
         else if (sub_count == 5'b10111)  pattern <= 4'b0000; 

         else                             pattern <= 4'b0000;
   
      end
   end
                  
               

endmodule               
